<?php

namespace Drupal\salesforce_commerce_stock_mapping\Plugin\SalesforceMappingField;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\salesforce\SObject;
use Drupal\salesforce_mapping\SalesforceMappingFieldPluginBase;
use Drupal\salesforce_mapping\Entity\SalesforceMappingInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Adapter for commerce_stock field.
 *
 * @Plugin(
 *   id = "CommerceStockLevel",
 *   label = @Translation("Commerce Stock Level")
 * )
 */
class CommerceStockLevel extends SalesforceMappingFieldPluginBase {

  /**
   * Stock Service Manager.
   */
  protected $stockServiceManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->stockServiceManager = $container->get('commerce_stock.service_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $pluginForm = parent::buildConfigurationForm($form, $form_state);

    $options = $this->getConfigurationOptions($form['#entity']);

    $pluginForm['drupal_field_value'] += [
      '#type' => 'select',
      '#options' => $options,
      '#empty_option' => $this->t('- Select -'),
      '#default_value' => $this->config('drupal_field_value'),
      '#description' => $this->t('Select commerce stock field'),
    ];

    return $pluginForm;

  }

  /**
   * Form options helper.
   */
  private function getConfigurationOptions(SalesforceMappingInterface $mapping) {

    $instances = $this->entityFieldManager->getFieldDefinitions(
      $mapping->get('drupal_entity_type'),
      $mapping->get('drupal_bundle')
    );

    $options = [];
    foreach ($instances as $key => $instance) {

      // Return only 'commerce_stock_level' field type
      if ($instance->getType() != 'commerce_stock_level') {
        continue;
      }

      $options[$key] = $instance->getLabel();
    }
    asort($options);
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function value(EntityInterface $entity, SalesforceMappingInterface $mapping) {
    return $this->stockServiceManager->getStockLevel($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function pullValue(SObject $sf_object, EntityInterface $entity, SalesforceMappingInterface $mapping) {
    $salesforce_stock = $sf_object->field($this->config('salesforce_field'));
    $commerce_stock = $this->stockServiceManager->getStockLevel(($entity));

    // Empty or non-numeric value means nothing to do here.
    if (empty($salesforce_stock) || !is_numeric($salesforce_stock)) {
      return;
    }

    // If values are equal already - nothing to do here.
    if ($salesforce_stock == $commerce_stock) {
      return;
    }

    return $salesforce_stock - $commerce_stock;
  }

}